/* ==============================================================
 *
 * Tis is example of using custom component "myEEPROM"
 * which saves application parameters in the EEPROM
 * between power-offs.
 *
 * saves upto 2047 bytes of data into EEPROM (+CRC8 1 byte)
 *
 * optional:
 *   CRC8 check upon EEPROM read,
 *   variable seed value for CRC operation,
 *   variable EEPROM address
 *   multiple instances of component allow to save several buffers
 *   Temperature check for operation in adverse conditions
 *
 *
 * Provided as-is, without any warranty or liability
 * under CREATIVE COMMONS - SHARE ALIKE license terms
 * ==============================================================
*/



#include <project.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h> //ceil

#if defined (__GNUC__)
    // Add an explicit reference to the floating point printf library
    // to allow the usage of floating point conversion specifiers. 
    // This is not linked in by default with the newlib-nano library. 
    asm (".global _printf_float");
#endif

//define pins for software UART
#define SW_UART_PORT       (1u) //Pin 1.4 - software defined pins
#define SW_UART_PIN        (4u) //Rx (yellow wire)
char strMsg1[128];              //char buffer for UART messages


    // can save packed (smaller size) or unpacked config buffer
    // uncomment #pragma ... lines for packed structure

    // this is test config data structure to be saved in EEPROM
    //#pragma pack(push, 1)   // make packed structure for storing config data
    typedef struct TConfig {
        char  Ch[48];       // some string  
        uint8 date;         //  
        int32 Current;      //
        float Voltage;      //   
    } ConfigBuffer;
    //#pragma pack(pop)       //back to the previous packing mode  

    struct TConfig Config;  // declare config buffer to hold all data to be saved in the EEPROM
    
    
//==============================================    
//          forvard declarations                         
//==============================================    
       
void SetDefaultConfigValues(void);  //initialize with some data
void ModifyConfigValues(void);      //modify data
void PrintConfig(char Msg[]);       //print UART report


void Initialize(void)
{
    SW_Tx_UART_StartEx(SW_UART_PORT, SW_UART_PIN);  // Initialize SW_Tx_UART 
        
    CyDelay(100);//must put to give time for Tx buffer flush else multiple output 
    SW_Tx_UART_PutString("\r\nmyEEPROM buffer storage component example\r\n\r\n"); 

    CyDelay(100);
    //sprintf(strMsg1, "sizeof(myStruct)=%d, \r\n", sizeof(Config) );
    //SW_Tx_UART_PutString(strMsg1);
    
    sprintf(strMsg1, "EEPROM_SIZE=%d, EEPROM_NUMBER_ROWS=%d\r\n\r\n", EEPROM_SIZE, EEPROM_NUMBER_ROWS );
    SW_Tx_UART_PutString(strMsg1);
        
    
    myEEPROM_1_Start( &Config, sizeof(Config) );//initialize myEEPROM with address and size of the storage buffer
    
    sprintf(strMsg1, "      EEPROM: initialized=%d, SizeOfBuffer=%d, start row=%d, rows used=%d \r\n",
                myEEPROM_1_Initialized(),
                myEEPROM_1_SizeOfBuffer(),
                myEEPROM_1_EEPROM_CONFIG_LOCATION,
                (uint8) ceil( (float)(myEEPROM_1_SizeOfBuffer()+1) / ((float) CYDEV_EEPROM_ROW_SIZE)) 
            );
    SW_Tx_UART_PutString(strMsg1);
    
}


int main()
{
    Initialize();                   //run initialization     
    
    
    
    SetDefaultConfigValues();       // populate config buffer with some default data
    //PrintConfig("        init:");
    
 
    //if valid EEPROM data available, recall values else use default
    if ( myEEPROM_1_recallConfig() ) //recall config from EEPROM
        PrintConfig(" recalled OK:");
    else
    {
        // recall failed: wrong/empty data - CRC did not match
        SW_Tx_UART_PutString("config recal: failed\r\n");  
        myEEPROM_1_storeConfig(); // store config buffer into EEPROM
        PrintConfig("    saved OK:");
    }    
    
    
    for(;;){}//loop forever
    
    //run program  for second time to observe recalled values!
}

//==============================================    
//  Functions                         
//==============================================    

void SetDefaultConfigValues(void) //initialize config buffer with some default values to be saved in EEPROM
{
        strcpy( Config.Ch, "my test app" );
        Config.date    = 15;
        Config.Current = -33;
        Config.Voltage = 4.095;
}

void ModifyConfigValues(void) //modify data in config buffer
{
        strcpy( Config.Ch, "--modified--" );
        Config.date    = 30;
        Config.Current = 1024;
        Config.Voltage = -0.01;
}

void PrintConfig(char Msg[]) //  print data values
{
    sprintf(strMsg1,"%s %s, %d, %d, %.3f \r\n", Msg,    Config.Ch,
                                                        Config.date,
                                                        Config.Current,
                                                        Config.Voltage
                                                    );
    SW_Tx_UART_PutString(strMsg1);
};


/* [] END OF FILE */


/*   
    //this is just another use of the component->
    
    SetDefaultConfigValues();       // populate config buffer with some default data
    
    if ( myEEPROM_1_storeConfig() ) // store config buffer into EEPROM
        PrintConfig("    saved OK:");
    else     
        SW_Tx_UART_PutString(" config save: failed\r\n");
 

    ModifyConfigValues();           // modify data in config buffer to see the difference
    PrintConfig("    modified:");   //show modified values
    
  
    if ( myEEPROM_1_recallConfig() ) //recall config from EEPROM
        PrintConfig(" recalled OK:");
    else     
        SW_Tx_UART_PutString("config recal: failed\r\n");
        
*/
